#include <qfile.h>
#include "gfcommon.h"
#include "gfprofile.h"

#ifndef GFCURVEH
#define GFCURVEH

/*

	GfCurve class 
	
	points start at 0 and go to NumPoints - 1
	

	modified for any number of columns, default is 2

*/

// for segment of curves routines special start values

/// start value for whole curve for segment methods
#define SEG_WHOLE_CURVE -1
/// start value for one point for segment methods
#define SEG_ONE_POINT -2
/// start value for last line for segment methods
#define SEG_LAST_LINE -3



class GfCurve
{

public:
	GfCurve(); 
	GfCurve(int); // constructor for more than 2 columns
	GfCurve(const GfCurve &); // copy constructor
	GfCurve(const GfCurve *); // copy constructor
	
	int copySegment(const GfCurve *,int,int); // copy of a segment -- not a constructor
	
	virtual ~GfCurve();
	
	int setPoint(int,int,float,float); // i,invec,x,y
	
	int appendPoint(int,float,float); // invec,x,y
	
	int insertPoint(int after,int iv,float x,float y);
	
	int removePoint(int);
	
	float pointX(int) const ;
	float pointY(int) const ;
	int pointInvec(int) const ;
	
	int setPatternLength(float );
	int setColor(int );
	int setLineType(int );
	int setLineWidth(int );
	
	
	int numPoints() const ;
	
	float patternLength() const ;
	
	int color() const ;
	int lineType() const ;
	int lineWidth() const ;
	
	void getMaxMin(float *xmax,float *xmin,float *ymax,float *ymin) const ;
	void getAverages(float *xave,float *yave) const ;
	
	void print(OSTREAM *);
	void format(OSTREAM *);
	int readFormat(QFile *in,char *line);
	
	int linearFit(float *slope,float *intercept,int start=SEG_WHOLE_CURVE,int end=0);
	void linearTransform(float a,float b,float c,float d,float e,float f);
	int reduceNumberPoints(int step);
	int smooth(int half_window);
		
	void operator=(const GfCurve &); // assignment operator
	
	int numColumns() const ;
	int xColumn() const ;
	int yColumn() const ;
	int row(int, float *) const ;
	int setRow(int,float *);
	
	int appendRow(int,float *);
	int insertRow(int after,int iv,float *);
	int removeRow(int);
	
	float *mallocRow();
	
	int setXColumn(int);
	int setYColumn(int);

	int sprintRow(char *text,int i,int vlen,int dlen,int line_fmt,char delim);
	

private:

	float *data;
	
	float xp(int) const ;
	float yp(int) const ;
	
	int setXp(int,float);
	int setYp(int,float);
	
	int xcol,ycol,numcols;
	
	int *invec;
	
	int numpoints;

	void init();
	
	void storage(int);
	
	int zeroRow(int);
	
	float patlen;
	
	int linetype;
	
	int c_color;
	
	int linewidth;
	
};

OSTREAM &operator<<(OSTREAM &os, GfCurve *m);
OSTREAM &operator<<(OSTREAM &os, GfCurve &m);


#endif // GfCurve
