/**********************************************************************

	--- Qt Architect generated file ---

	File: GfWidget.cpp
	Last generated: Thu Jun 17 16:50:15 1999

 *********************************************************************/
#define GFMAIN

#include "GfWidget.h"
#include <qcolor.h>
#include <qfiledialog.h>
#include <qfile.h>
#include <qmessagebox.h>
#include <qkeycode.h>
#include <qprinter.h>
#include <qpaintdevicemetrics.h>
#include "../gfprofilewin/GfProfileWin.h"
#include "../gfscalewin/GfScaleWin.h"
#include "../gfcommentswin/GfCommentsWin.h"
#include "../gfcurvewin/GfCurveWin.h"
#include "../gfplotdefswin/GfPlotDefsWin.h"
#include "../gfnumcolswin/GfNumcolsWin.h"
#include "../gfdatatranswin/GfDataTransWin.h"
#include "../editcurvewin/EditCurveWin.h"


#define Inherited GfWidgetData

GfWidget::GfWidget
(
	QWidget* parent,
	const char* name
)
	:
	Inherited( parent, name )
{
	setBackgroundColor(white); 
	
	popup_enabled = 1;
	
	filepopup = new QPopupMenu();
	
	filepopup->insertItem("Open...",this,SLOT(open()),CTRL+Key_O);
	filepopup->insertItem("Append...",this,SLOT(append()),CTRL+Key_A);
	filepopup->insertSeparator();
	filepopup->insertItem("Close",this,SLOT(closeWin()),CTRL+Key_W);
	filepopup->insertItem("Save...",this,SLOT(saveAs()),CTRL+Key_S);
	filepopup->insertSeparator();
	filepopup->insertItem("File Format...",this,SLOT(profile()));
	filepopup->insertSeparator();
	filepopup->insertItem("Page Setup...",this,SLOT(plotFormat()));
	filepopup->insertItem("Save As PostScript...",this,SLOT(saveAsPostscript()),CTRL+Key_P);
	filepopup->insertItem("Print...",this,SLOT(print()));
	filepopup->insertSeparator();
	filepopup->insertItem("Quit",this,SLOT(quit()),CTRL+Key_Q);
	
	settingspopup = new QPopupMenu();
	
	settingspopup->insertItem("File",filepopup);
	
	settingspopup->insertItem("Auto Scale",this,SLOT(autoScale()));
	settingspopup->insertSeparator();
	settingspopup->insertItem("Rescale...",this,SLOT(rescale()),CTRL+Key_R);
	settingspopup->insertItem("Comment...",this,SLOT(comments()));
	settingspopup->insertItem("Curves Format...",this,SLOT(curveFormat()));
	settingspopup->insertItem("Data Transforms...",this,SLOT(dataTrans()));
	settingspopup->insertSeparator();
	//settingspopup->insertItem("Quit",qApp,SLOT(quit()));
	settingspopup->insertItem("Close",this,SLOT(closeWin()));
	
	selection_curve = -1; // no selection
	
	use_pixmap_redraw = GF_NO;
	redraw_pixmap = NULL;
	
}


GfWidget::~GfWidget()
{
	if(redraw_pixmap != NULL)
		delete redraw_pixmap;
		
	delete filepopup;
	delete settingspopup;
}


void GfWidget::setGfFile(const GfFile &gff)
{
	gffile = gff;
	gfscale = gff.scale();
	plotdefs = gff.plotDefs();
	erase();
	draw(this);
	
}

void GfWidget::setScale(const GfScale &s)
{
	gffile.setScale(s);
	erase();
	draw(this);
}

void GfWidget::setProfile(const GfProfile &p)
{
	gffile.setProfile(p);
}

void GfWidget::paintEvent ( QPaintEvent * )
{
	draw(this);
}

void GfWidget::resizeEvent(QResizeEvent *)
{
	erase();
	draw(this);           
}


void GfWidget::draw(QPaintDevice *w)
{              
#ifdef DEBUG1
QTime time;
int t0;

	time.start();
#endif	
	if(use_pixmap_redraw == GF_YES)
	{
		bitBlt(this,0,0,redraw_pixmap);
	}
	else
	{			
				
		plotdefs = gffile.plotDefs();
	
		gfscale = gffile.scale();
        
      		QPainter paint(w);
	
		gfdraw.initPaint(&paint,width(),height(),&plotdefs,&gfscale);
	
		gffile.draw(&gfdraw);
	
		if(selection_curve > -1)
			gffile.drawSelection(&gfdraw,selection_curve,selection_start,selection_end);
		
		gfdraw.done();
	}
	
#ifdef DEBUG1
	t0 = time.elapsed();
	
	if(use_pixmap_redraw == GF_YES)
	{
		cout<<"pixmap time = "<<t0<<"\n";
	}
	else
	{
		cout<<"draw time = "<<t0<<"\n";
	}
#endif	
}

int GfWidget::setPixmapRedraw(int state)
{
/*
	if we are using PixmapRedraw, only the methods:
	
		appendPointDraw and appendRowDraw
		
	will result in a change in the display.
	
	turn pixmapRedraw off to make any other changes to display.
*/

	if(state == GF_YES && use_pixmap_redraw == GF_NO)
	{
		setFixedSize(width(),height()); 
		
		redraw_pixmap = new QPixmap(width(),height());
		
		use_pixmap_redraw = GF_YES;
		
		plotdefs = gffile.plotDefs();
	
		gfscale = gffile.scale();
        
		QPainter paint;
			
		paint.begin(redraw_pixmap,this);
		
		paint.fillRect(0,0,width(),height(),QBrush(backgroundColor()));
		
		gfdraw.initPaint(&paint,width(),height(),&plotdefs,&gfscale);
	
		gffile.draw(&gfdraw);
		
		gfdraw.done();
			
		paint.end();
		
		return(GF_OK);
	}
		
	if(state == GF_NO && use_pixmap_redraw == GF_YES)
	{
		use_pixmap_redraw = GF_NO;
		delete redraw_pixmap;
		redraw_pixmap = NULL;
		
		setMaximumSize(2048,2048);
		setMinimumSize(50,50);
		
		return(GF_OK);
	}
		
	return(GF_BAD);
}	

void GfWidget::setSelection(int curve,int start,int end)
{
	selection_curve = curve;
	selection_start = start;
	selection_end = end;
}

void GfWidget::mousePressEvent ( QMouseEvent *m )
{
int curve,point;

	if(popup_enabled == 1)
	{
		settingspopup->exec(m->globalPos());
	}
	else
	{
		emit gfWidgetClicked(m);
		
		if(gffile.closestPoint(m->x(),height()-m->y(),&gfdraw,&curve,&point) == GF_OK)
		{
			emit pointClicked(curve,point);
#ifdef DEBUG1
			cout << "curve,point = "<<curve<<" , "<<point<<"\n";
#endif			
		}
	}

}

void GfWidget::saveAsPostscript()
{
int p;

	GfPlotDefs pd;
			
	pd = gffile.plotDefs();
	
	QString exten(".ps");
	if(pd.encapsulated() == GF_YES)
	{
		exten = ".eps";
	}
			

	QString name(gffile.fileName());
	
	if(!name.contains("+")) // not appended
	{
		p = name.find('.');
		name.remove(p,10);
		name.append(exten);
	}
	else
	{
		name = "";
	}
	
	exten.prepend("*");
	
	QFileDialog qfd(0,exten,this,0,TRUE);
	
	qfd.setSelection(name);
	
	qfd.setCaption("Save as Postscript");
	
	if(qfd.exec() == 0) return;
	
	QString fileName = qfd.selectedFile();
		
	if ( !fileName.isNull() ) 
	{                
        	QFile qf(fileName);
		
		if(qf.exists() )
		{
			if( QMessageBox::warning( this, "Gf",
                                  "File Exists, Overwrite?",
                                  "No", "Yes", 0,
                                  0, 0 ) == 0 )
			{
				return;
			}
		}
		
		if(qf.open(IO_WriteOnly) == TRUE)
		{
			GfScale s;
			
			s = gffile.scale();
			
			GfDraw gfd;			
			
			if(gfd.initPS(&qf,&pd,&s) == GF_BAD)
			{
				errorBox("Bad Plot Size");
				return;
			}
			
			gffile.draw(&gfd);
						
			gfd.done();
			
			qf.close();
			
			errorBox("PS written to file");
		}
		else
		{
			errorBox("File Open failed");
		}	
	}
	
}
void GfWidget::print()
{
	GfPlotDefs pd;
			
	pd = gffile.plotDefs();
	
	QPrinter prt;
	
	prt.setPageSize(QPrinter::Letter);
	prt.setCreator("GfQt");
	prt.setDocName(gffile.fileName());
	
	if(pd.rotatePlot() == GF_YES) // portrait
	{
		prt.setOrientation(QPrinter::Portrait);
	}
	else
	{
		prt.setOrientation(QPrinter::Landscape);
	}
			
	if( prt.setup() ) 
	{                
 		GfScale s;
			
		s = gffile.scale();
			
		GfDraw gfd;	
		
		QPainter paint;
		
		QPaintDeviceMetrics qpdm(&prt);
		
/* 		QString text;
		
		text.sprintf("width = %i, height = %i mm",
			qpdm.widthMM(),qpdm.heightMM());
			
		errorBox(text);
 */		
		paint.begin(&prt);
		
		int w_mm = qpdm.widthMM();
		int h_mm = qpdm.heightMM();
			
		int w,h;
		
		w = (int)(((float)w_mm)*72.0/25.4);
		h = (int)(((float)h_mm)*72.0/25.4);
		
              	paint.setWindow(0,0,w,h);
		
		//int w = qpdm.width();
		//int h = qpdm.height();		
			
		if(gfd.initPaintPrinter(&paint,w,h,&pd,&s) == GF_BAD)
		{
			errorBox("Internal Error");
			return;
		}
			
		gffile.draw(&gfd);
		
		gfd.done();

		paint.end();
		
		errorBox("Printing Complete");
	}
	
}

void GfWidget::open()
{	
	QString fileName = QFileDialog::getOpenFileName(0,"*.gf",this,"Open");
	
	open_append(fileName,GF_NO);	
}

void GfWidget::open_append(QString fileName, int append)
{
int ncols,xcol,ycol;

	if ( !fileName.isNull() ) 
	{                
        	QFile qf(fileName);
		
		if(qf.open(IO_ReadOnly) == TRUE)
		{
			ncols = gffile.checkNumCols(&qf);
		
			if(ncols > 2)
			{
				GfNumcolsWin gfnumcol(ncols,&xcol,&ycol);
				
				if(gfnumcol.exec() == GF_YES)
				{
					if(gffile.read(&qf,xcol,ycol,append) == GF_BAD)
					{
						errorBox("Read failed");
					}
				}
				else
				{
					return;  // cancel clicked
				}
			
			}
			else
			{
				if(gffile.read(&qf,1,2,append) == GF_BAD)
				{
					errorBox("Read failed");
				}
			}	
			
			gfscale = gffile.scale();
			
			erase();
			draw(this);
			
			emit fileChanged();

		}
		else
		{
			errorBox("File Open failed");
		}
	}
}

void GfWidget::errorBox(const char *msg)
{
	QMessageBox mb("Gf",msg,QMessageBox::NoIcon,
		QMessageBox::Ok | QMessageBox::Default,0,0,this);
		
	mb.exec();
}

void GfWidget::append()
{	
	QFileDialog qfd(0,"*.gf",this,0,TRUE);
	
	qfd.setCaption("Append");
	
	if(qfd.exec() == 0) return;
	
	QString fileName = qfd.selectedFile();
	
	open_append(fileName,GF_YES);	
	
}

int GfWidget::read(QFile *qf,int xcol,int ycol,int append)
{
	return(gffile.read(qf,xcol,ycol,append));
}

void GfWidget::saveAs()
{
	QString name(gffile.fileName());
	
	if(name.contains("+")) // appended
	{
		name = "";
	}
	
	QString fileName = QFileDialog::getSaveFileName(name,"*.gf",this);
	
	if ( !fileName.isNull() ) 
	{                
        	QFile qf(fileName);
		
		if(qf.exists() )
		{
			if( QMessageBox::warning( this, "Gf",
                                  "File Exists, Overwrite?",
                                  "No", "Yes", 0,
                                  0, 0 ) == 0 )
			{
				return;
			}
		}
		
		if(qf.open(IO_WriteOnly) == TRUE)
		{		 	
			QApplication::setOverrideCursor( waitCursor );
			
			gffile.write(&qf);
			
    			QApplication::restoreOverrideCursor();
			
			emit fileChanged();
			
			QFile wchk(fileName);
			
			if( wchk.exists() == FALSE)
			{
				QMessageBox mb("Save As","File not Written",QMessageBox::Warning,
					QMessageBox::Ok | QMessageBox::Default,0,0,this);
		
				mb.exec();
			}
		}
		else
		{
			QMessageBox mb("Save As","Can't write file",QMessageBox::Warning,
				QMessageBox::Ok | QMessageBox::Default,0,0,this);
		
			mb.exec();
		}			
		
	}
}

void GfWidget::quit()
{
	emit popupQuit(1);
}

void GfWidget::closeWin()
{
	emit popupQuit(2);
}

void GfWidget::rescale()
{
/*
	gffile.autoScale();
	gfscale = gffile.scale();
	erase();
	draw(this);
*/
GfScaleWin *win;
	win = new GfScaleWin(&gffile);	
	connect(win,SIGNAL(scaleChanged()),this,SLOT(redraw()));	
	win->exec();	
	delete win;
}

void GfWidget::redraw()
{
	erase();
	draw(this);
}

void GfWidget::comments()
{
GfCommentsWin *win;

	win = new GfCommentsWin(&gffile);	
	connect(win,SIGNAL(commentsChanged()),this,SLOT(redraw()));	
	win->exec();	
	delete win;
}

void GfWidget::profile()
{
GfProfileWin *win;

	win = new GfProfileWin(&gffile);
	win->exec();
	delete win;

}

void GfWidget::curveFormat()
{
GfCurveWin *win;

	win = new GfCurveWin(&gffile);
	connect(win,SIGNAL(curveChanged()),this,SLOT(redraw()));
	win->exec();
	delete win;
}

void GfWidget::plotFormat()
{
GfPlotDefsWin *win;

	win = new GfPlotDefsWin(&gffile);
	connect(win,SIGNAL(plotDefsChanged()),this,SLOT(redraw()));
	win->exec();
	delete win;
}

void GfWidget::dataTrans()
{
GfDataTransWin *win;

	win = new GfDataTransWin(&gffile);
	connect(win,SIGNAL(dataChanged()),this,SLOT(redraw()));
	win->exec();
	delete win;
}

void GfWidget::editCurves()
{
	if(gffile.numPoints() == 0)
	{
		gffile.appendPoint(0,0,0.,0.);
	}
	
	EditCurveWin win(&gffile);

	connect(&win,SIGNAL(dataChanged()),this,SLOT(redraw()));
	
	win.exec();

}	

void GfWidget::enablePopupMenu()
{
	// note: filepopup and settingspopup must not be in use elsewhere
	//settingspopup->insertItem("File...",filepopup,-1,0);
	//settingspopup->insertSeparator();
	//settingspopup->insertItem("Close",this,SLOT(closeWin()));
	popup_enabled = 1;
}

void GfWidget::disablePopupMenu()
{
	popup_enabled = 0;
}

QPopupMenu *GfWidget::filePopupMenu()
{
	// first we have to stop using the filepopup
	
	settingspopup->removeItemAt(0);
	
	// and remove quit from the settings menu
	settingspopup->removeItemAt(settingspopup->count()-1);
	settingspopup->removeItemAt(settingspopup->count()-1);	
	
	return(filepopup);
}

QPopupMenu *GfWidget::settingsPopupMenu()
{
	popup_enabled = 0;	
	
	return(settingspopup);
}

QString GfWidget::fileName() const 
{
	return(gffile.fileName());
}

int GfWidget::checkNumCols(QFile *qf)
{
	return(gffile.checkNumCols(qf));
}

GfCurve GfWidget::curveSegment(int curve,int start,int end) const 
{
GfCurve ans;

	ans=gffile.curveSegment(curve,start,end);
	
	return(ans);

}

int GfWidget::deleteCurveSegment(int curve,int start,int end)
{
GfCurve c,cbegin,cend;

	if(start == SEG_WHOLE_CURVE) // remove whole curve
	{
		return(gffile.removeCurve(curve));
	}

	c = gffile.curve(curve);
	
	if(c.numPoints() > 0)
	{
		if( start > 0)
		{
			if(cbegin.copySegment(&c,0,start) == GF_BAD)
			{
				return(GF_BAD);
			}
		}
		
		if( end < (c.numPoints()-1) )
		{
			if(cend.copySegment(&c,end,c.numPoints()-1) == GF_BAD)
			{
				return(GF_BAD);
			}
		}
		
		if(cbegin.numPoints() > 1)
		{
			gffile.setCurve(curve,cbegin);
		}
		else
		{
			gffile.removeCurve(curve);
		}
		
		if(cend.numPoints() > 1)
			gffile.appendCurve(cend);
		
		return(GF_OK);	
	}
	else
		return(GF_BAD);

}
int GfWidget::appendCurve(const GfCurve *c,const GfScale &s)
{
	int n = gffile.numPoints();
	
	gffile.appendCurve(c);
	
	if(n == 0) 
	{
		setScale(s);
	}
	
	return(GF_OK);

}

int GfWidget::appendPoint(int c,int iv,float x,float y)
{

	return(gffile.appendPoint(c,iv,x,y));

}

int GfWidget::appendRow(int c,int iv,float *row)
{

	return(gffile.appendRow(c,iv,row));

}

int GfWidget::appendPointDraw(int c,int iv,float x,float y)
{
	if(gffile.appendPoint(c,iv,x,y) != GF_BAD)
	{
	
      		QPainter paint(this);
	
		gfdraw.initPaint(&paint,width(),height(),&plotdefs,&gfscale);
	
		gffile.drawSelection(&gfdraw,c,SEG_LAST_LINE,0,0); // last 2 pts
		
		gfdraw.done(); // draw selection does not call done

		if(use_pixmap_redraw == GF_YES)
		{
      			QPainter paint;
			
			paint.begin(redraw_pixmap,this);
	
			gfdraw.initPaint(&paint,width(),height(),&plotdefs,&gfscale);
	
			gffile.drawSelection(&gfdraw,c,SEG_LAST_LINE,0,0); // last 2 pts
			
			paint.end();
			
			gfdraw.done();
		}
					
		return(GF_OK);
	}
	else
	{
		return(GF_BAD);
	}

}

int GfWidget::appendRowDraw(int c,int iv,float *row)
{
	if(gffile.appendRow(c,iv,row) != GF_BAD)
	{
	
      		QPainter paint(this);
	
		gfdraw.initPaint(&paint,width(),height(),&plotdefs,&gfscale);
	
		gffile.drawSelection(&gfdraw,c,SEG_LAST_LINE,0,0); // last 2 pts
		
		gfdraw.done();

		if(use_pixmap_redraw == GF_YES)
		{
      			QPainter paint;
			
			paint.begin(redraw_pixmap,this);
	
			gfdraw.initPaint(&paint,width(),height(),&plotdefs,&gfscale);
	
			gffile.drawSelection(&gfdraw,c,SEG_LAST_LINE,0,0); // last 2 pts
			
			paint.end();
			
			gfdraw.done();
		}
					
		return(GF_OK);	
	}
	else
	{
		return(GF_BAD);
	}

}

void GfWidget::autoScale()
{
	gffile.autoScale();
	erase();
	draw(this);
}

GfScale GfWidget::scale() const 
{
	GfScale s;
	
	s = gffile.scale();
	
	return(s);
}

GfPlotDefs GfWidget::plotDefs() const 
{
	GfPlotDefs pd;
	
	pd = gffile.plotDefs();
	
	return(pd);
}

GfFile GfWidget::gfFile() const
{
	return(gffile);
}

int GfWidget::getMaxMin(int curve,float *xmax,float *xmin,float *ymax,float *ymin) const 
{
	return(gffile.getMaxMin(curve,xmax,xmin,ymax,ymin));
}

void GfWidget::setPlotDefs(const GfPlotDefs &pd)
{
	gffile.setPlotDefs(pd);
}

int GfWidget::setCurvePlotColumns(int curve,int xcol,int ycol)
{
	return(gffile.setCurvePlotColumns(curve,xcol,ycol));
}

int GfWidget::numCurves() const 
{
	return(gffile.numCurves());
}

int GfWidget::numPoints() const 
{
	return(gffile.numPoints());
}

void GfWidget::setInFileComments(const char *text)
{
	gffile.setInFileComments(text);
}

// support for OverlayGfDraw usage.

/**
	This function is for use with overlays.
*/
void GfWidget::redrawLastPoint(GfDraw *gfd,int c)
{

	gffile.drawSelection(gfd,c,SEG_LAST_LINE,0,0); // last 2 pts

}

void GfWidget::draw(GfDraw *gfd)
{
	gffile.draw(gfd);
}

